##################################################
## several EM functions for estimating f0 and f1.
##################################################

##################################################
## EM for fitting two component normal mixture model. 
## This is for single condition only.
## Note I force that sd2>=sd1 for var.eq=FALSE
##################################################
mix.2norm <- function(Y, mu1, pi.grp2=0.5, nmaxiter=100, TOL=1e-10*length(Y),
                      var.eq=TRUE, verbose=TRUE) {
  
  n <- length(Y)
  est.mu1 <- FALSE
  if(missing(mu1))
    est.mu1 <- TRUE

  ## get initial values
  mu0 <- 0
  if(missing(mu1)) {
    tmp <- quantile(Y, c((1-pi.grp2)/2,1-pi.grp2/2))
    mu1 <- tmp[2]
  }
  tmp <- weights <- matrix(0, length(Y), 2)
  loglik.old <- 0
  sd0 <- sd1 <- sd(Y)
  ## EM iteration
  for(i in 1:nmaxiter) {
    ## E-step: calculate responsibilities
    weights[,1] <- dnorm(Y, mu0, sd0)
    weights[,2] <- dnorm(Y, mu1, sd1)
    tmp <- sweep(weights, 2, c(1-pi.grp2, pi.grp2), "*")
    ll<- rowSums(tmp)
    gamma <- sweep(tmp, 1, ll, "/")
    loglik <- sum(log(ll))
    if(abs(loglik-loglik.old) < TOL)
      break
    loglik.old <- loglik
    
    ## M-step: 
    s <- sum(gamma[,2])
    pi.grp2 <- s/n
    
    ## update mu's and sd's
    mu0 <- 0 ##sum(gamma[,1]*Y)/(n-s)
    if(est.mu1)
      mu1 <- sum(gamma[,2]*Y)/s
    if(var.eq) {
      sd0 <- sd1 <- sqrt((sum(gamma[,1]*(Y-mu0)^2)+sum(gamma[,2]*(Y-mu1)^2))/n)
    }
    else {
      sd0 <- sqrt(sum(gamma[,1]*(Y-mu0)^2)/(n-s))
      sd1 <- sqrt(sum(gamma[,2]*(Y-mu1)^2)/s)
      if(sd1<sd0) { ## the second group must have larger variance
        sd0 <- sd1 <- sqrt((sum(gamma[,1]*(Y-mu0)^2)+sum(gamma[,2]*(Y-mu1)^2))/n)
      }      
    }
    if(verbose) 
      cat(i, "loglik=", loglik,": mu0=", mu0, ", mu1=",
          mu1, "sd0=", sd0, "sd1=",sd1,"\n")
  }
  
  ## return a list
  list(mu0=mu0, mu1=mu1, tau2.0=sd0^2, tau2.1=sd1^2, pi.grp2=pi.grp2)
}


#########################################################################
## Yet another function to do mixture model using EM.
## init.idx is T/F vector for observations in first group
## sd0 is a vector of SD for dat
## -- this one assumes both group have the same tau
#########################################################################
mix.2norm.var <- function(dat, sigma2, mu1, pi.grp2, nmaxiter=100, TOL=1e-4, verbose=TRUE) {
  ## remove those with variance=0
  idx <- sigma2>0
  dat <- dat[idx]
  sigma2 <- sigma2[idx]
  n <- length(dat)
  
  ## calculate initial values #####
  ##  group means
  mu0 <- 0
  est.mu1 <- FALSE
  if(missing(mu1)) {
    est.mu1 <- TRUE
    mu1 <- quantile(dat, 1-pi.grp2/2)
  }
  tau2 <- 0.1
  
  ##### start EM #######
  loglik.old <- 0
  tmp <- weights <- matrix(0, n, 2)
  mu1.new <- mu1
  for(i in 1:nmaxiter) {
    tmpsd <- sqrt(sigma2+tau2)
    ## E-step:
    ## 1. calculate E[b|data]
    weights[,1] <- dnorm(dat, mu0, tmpsd)
    weights[,2] <- dnorm(dat, mu1, tmpsd)
    tmp[,1] <- weights[,1]*(1-pi.grp2)
    tmp[,2] <- weights[,2]*pi.grp2
    ll<- rowSums(tmp)
    gamma <- sweep(tmp, 1, ll, "/") ## now gamma are responsibilities
    ## 2. calculate E[mu_i], E[mu_i*b_i] and E[mu_i^2*b_i]
    denom <- 1/sigma2 + 1/tau2
    E.h.X <- gamma[,2]*(dat/sigma2+mu1/tau2)/denom
    E.h <- E.h.X + gamma[,1]*(dat/sigma2)/denom
    E.h2.X <- gamma[,2]*(((dat/sigma2+mu1/tau2)/denom)^2+1/denom)
    E.h2 <- E.h2.X + gamma[,1]*(((dat/sigma2)/denom)^2 + 1/denom)
    
    ## M-step:
    ss <- sum(gamma[,2])
    ## update pi
    pi.grp2.new <- ss/n
    ## update mu
    if(est.mu1) 
      mu1.new <- sum(E.h.X) / ss
    ## update tau - assume both group have same tau - check results
    tau2.new <- sum(E.h2+mu1.new^2*gamma[,2]-2*mu1.new*E.h.X)/n
    if(verbose)
      cat("iter", i, pi.grp2.new, mu1.new, tau2.new, "\n")
    ## check convergence
    dd <- sqrt((mu1.new-mu1)^2 + (tau2.new-tau2)^2)
    if(dd<TOL)
      break
    
    mu1 <- mu1.new
    tau2 <- tau2.new
    pi.grp2 <- pi.grp2.new
  }
  list(mu0=mu0, mu1=mu1, tau2.0=tau2, tau2.1=tau2, pi=pi.grp2, coverget=i<nmaxiter, gamma=gamma[,2])
}


#########################################################################
## Yet another function to do mixture model using EM.
## This assumes different tau2's for two groups.
#########################################################################
mix.2norm.var2 <- function(dat, sigma2, pi.grp2, nmaxiter=100, TOL=1e-6, verbose=TRUE) {
  n <- length(dat)
  ## calculate initial values #####
  ##  group means
  mu0 <- 0
  mu1 <- quantile(dat, 1-pi.grp2/2)
  tau2.0 <- tau2.1 <- 0.1

  ##### start EM #######
  loglik.old <- 0
  tmp <- weights <- matrix(0, n, 2)
  for(i in 1:nmaxiter) {
    tmpsd0 <- sqrt(sigma2+tau2.0)
    tmpsd1 <- sqrt(sigma2+tau2.1)
    
    ## E-step:
    ## 1. calculate E[b|data]
    weights[,1] <- dnorm(dat, mu0, tmpsd0)
    weights[,2] <- dnorm(dat, mu1, tmpsd1)
    tmp[,1] <- weights[,1]*(1-pi.grp2)
    tmp[,2] <- weights[,2]*pi.grp2
    ll<- rowSums(tmp)
    gamma <- sweep(tmp, 1, ll, "/") ## now gamma are responsibilities
    ## 2. calculate E[mu_i], E[mu_i*b_i] and E[mu_i^2*b_i]
    denom0 <- 1/sigma2 + 1/tau2.0
    denom1 <- 1/sigma2 + 1/tau2.1
    E.h.X <- gamma[,2]*(dat/sigma2+mu1/tau2.1)/denom1
    E.h <- E.h.X + gamma[,1]*(dat/sigma2)/denom0
    E.h2.X <- gamma[,2]*(((dat/sigma2+mu1/tau2.1)/denom1)^2+1/denom1)
    E.h2 <- E.h2.X + gamma[,1]*(((dat/sigma2)/denom0)^2 + 1/denom0)
    
    ## M-step:
    ss <- sum(gamma[,2])
    ## update pi
    pi.grp2.new <- ss/n
    ## update mu
    mu1.new <- sum(E.h.X) / ss
    ## update tau - assume two groups have different tau
    ## check tau2.0.new 
    tau2.0.new <- sum(E.h2-E.h2.X) / (n-ss)
    tau2.1.new <- sum(E.h2.X+mu1^2*gamma[,2]-2*mu1*E.h.X)/ss
    
    if(verbose)
      cat("iter", i, pi.grp2.new, mu1.new, tau2.0.new, tau2.1.new, "\n")
    ## check convergence
    dd <- sqrt((mu1.new-mu1)^2 + (tau2.0.new-tau2.0)^2 + (tau2.1.new-tau2.1)^2)
    if(dd<TOL)
      break
    mu1 <- mu1.new
    tau2.0 <- tau2.0.new
    tau2.1 <- tau2.1.new
    pi.grp2 <- pi.grp2.new
  }
  list(mu0=mu0, mu1=mu1, tau2.0=tau2.0, tau2.1=tau2.1, pi=pi.grp2,
       converge=i<nmaxiter, gamma=gamma[,2])
}

