### Estimate parameters using EM

est.param <- function(candidates, cfg) {
  Y <- candidates$signal
  s2 <- candidates$vars ## note this has been shrunk
  allchr <- candidates$allchr
  allpos <- candidates$allpos
  distn <- candidates$distn
  sep <- cfg$peak$candidateLength
  
  ## check inputs
  if(!is.matrix(Y))
    Y <- matrix(Y)
  if(is.matrix(s2)) {
    if( !all(dim(Y)==dim(s2)) )
      stop("Input data and variance should have the same dimensions.\n")
  }

  nobs <- dim(Y)[1]
  ncond <- dim(Y)[2]
  
  ## step 0: compute densities in two groups, lr is the sufficient statistics
  dens <- compute.density(Y, s2, distn) 
  ylr <- dens$lden1 - dens$lden0

  ## step 1: get liklihoods ratios for each candidate peak by marginalizing peaks 
  lr <- matrix(0, nrow=nobs, ncol=ncond)
  counter <- 0
  idx <- split(1:length(allchr), allchr)
  for(i in seq(along=idx)) {
    tmpidx <- idx[[i]]
    if(length(tmpidx)<4) next
    dtmp <- .Call("calc_Candidate_Lr", ylr[tmpidx,,drop=FALSE], as.integer(allpos[tmpidx]),
                  as.integer(sep), as.integer(cfg$peak$bumpLength))
    dtmp <- matrix(dtmp, byrow=TRUE, ncol=ncond)
    if(nrow(dtmp)==0)
      next
    lr[counter+(1:nrow(dtmp)),] <- dtmp
    counter <- counter+nrow(dtmp)
  }
  lr <- lr[1:counter,,drop=FALSE]
  ## remove the problematic candidates
  lr <- lr[lr[,1]!=-1000,,drop=FALSE]
  ## step 2: estimate parameters using EM,
  ## should I only use the most obvious peaks from each conditions.
  ## Use big peaks give more stable EM estimates. Otherwise it sometimes stuck at local maxima.
  ## Can this artificially increase correlation??? This will underestimate sita.
##   if(nrow(lr)>500) {
##     lridx <- NULL
##     for(i in 1:ncond) {
##       lridx <- c(lridx,order(lr[,i], decreasing=TRUE)[1:500])
##     }
##     lridx <- unique(lridx)
##     lr <- lr[lridx,]
##   }
  res <- est(lr, candidates$pi)
  ## get pi for single condition
  res$pi1 <- rep(0, ncond)
  for(i in 1:ncond)
    res$pi1[i] <- est1(lr[,i], candidates$pi)
  
  res
}


###########################################################
## function to estmate q and sita
## modified: add condition specific groups
###########################################################
est <- function(lr, pi, nmaxiter=50) {
  nbumps <- nrow(lr)
  ncond <- ncol(lr)
  
#  q <- rep(0.5,ncond) 
#  sita <- c(0.5, 0.5)
#  sita <- c(0.99, 0.01)
  sita <- runif(2)
  sita <- sita/sum(sita)
  q <- runif(ncond)
  ## rescale sita - do we need this??? Seems yes.
  sita[-1] <- sita[-1]*pi
  sita[1] <- 1-sum(sita[-1])

  ## initialize some parameters
  f1q1 <- fyij.z <- matrix(0, nrow=nbumps, ncol=ncond)
  fyi.z <- rep(0, nbumps)
  for(iter in 1:nmaxiter) { ## EM iteration
    ## cat(iter, ", ")
    lq1 <- log(q)
    lq0 <- log(1-q)
    lsita <- log(sita)
    ## compute marginal lr: f(Yij|Z=1)-f(Yij|Z=0)=lr_ij*qj + (1-qj)
    f1q1 <- sweep(lr, 2, lq1, "+")
    fyij.z <- matrix(sweep(f1q1, 2, lq0, Raddlog), ncol=ncond)

    ## marginalize for conditions, f(Yi|Z=1)
    fyi.z <- rowSums(fyij.z)

    ## complete marginal f(Yi) - need to be careful here:
    ## CANNOT marginalize f(Yij) then multiply, because the hidden states are not independent.
    ##  fyi <- rowSums(fyij) - this is wrong!!!
    ptmp <- fyi.z+lsita[2] # this is LR(yi|Z=1)P(Z=1)
    fyi <- Raddlog(ptmp, rep(lsita[1], nbumps))
    ## E-step
    ## E[Zi=k|data]
    mu <- exp(ptmp-fyi)
    mu[mu>1] <- 1 ## correct for numeric problem
    ## E[Zi=k, Xij=1|data]
    lambda <- matrix(0, nrow=nbumps, ncol=ncond)
    for(icond in 1:ncond) {
      ## numerator
      p.datai <- rowSums(fyij.z[,-icond,drop=FALSE])+f1q1[,icond]+lsita[2]
      lambda[,icond] <- exp(p.datai-fyi)
    }
    lambda[lambda>1] <- 1
    ## M-step
    ## update sita
    ss <- mean(mu)
    sita.new <- c(1-ss, ss)
    ## rescale sita
    sita.new[-1] <- sita.new[-1]*pi
    sita.new[1] <- 1-sum(sita.new[-1])
    ## update q 
    q.new <- colMeans(lambda)/ss
    ## check convergence
    change <- sqrt(mean((q.new-q)^2)+mean((sita.new-sita)^2))
    if(change < 1e-3) {
      q <- q.new
      sita <- sita.new
      q[q>1] <- 1
      q[q<0] <- 0
      break
    }
    q <- q.new
    sita <- sita.new
    ## correct for numeric problem
    q[q>1] <- 1
    q[q<0] <- 0
  }

  ## return
#  sita[-1] <- sita[-1]*pi*1.5
#  sita[1] <- 1-sum(sita[-1])
  list(q=q, sita=sita)
}

###########################################################
## estimate pi for single condition
###########################################################
est1 <- function(lr1, pi, nmaxiter=50) {
  sita <- runif(2)
  sita <- sita/sum(sita)
  ## rescale sita - do we need this??? Seems yes.
  sita[-1] <- sita[-1]*pi
  sita[1] <- 1-sum(sita[-1])

  ## initialize some parameters
  for(iter in 1:nmaxiter) { ## EM iteration
    ## cat(iter, ", ")
    ## E-step: posterior probabilities
    lsita <- log(sita)
    tmp <- lr1+lsita[2]
    mu <- exp(tmp-sapply(tmp, function(x) Raddlog(x,lsita[1])))
    ## M-step
    ## update sita
    ss <- mean(mu)
    sita.new <- c(1-ss, ss)
    ## rescale sita
    sita.new[-1] <- sita.new[-1]*pi
    sita.new[1] <- 1-sum(sita.new[-1])
    ## check convergence
    change <- sqrt(mean((sita.new-sita)^2))
    if(change < 1e-3) {
      sita <- sita.new
      break
    }
    sita <- sita.new
  }

  ## return
  sita[2] #*pi
}


