## some utility functions

###############################################
## filter taking care of edges
###############################################
myfilter <- function(x,filter,...){
  if(length(x)<length(filter)) ## return mean
    return(rep(mean(x), length(x)))

  res=filter(x,filter,...)
  ##now fill out the NAs
  M=length(filter)
  N=(M- 1)/2
  L=length(x)
  for(i in 1:N){
    w=filter[(N-i+2):M]
    y=x[1:(M-N+i-1)]
    res[i] = sum(w*y)/sum(w)
    
    w=rev(w)
    ii=(L-(i-1))
    y=x[(ii-N):L]
    res[ii]<-sum(w*y)/sum(w)
  }
  return(res)
}

###############################################
## a function to smooth piece by piece
## len is a vector for the length of each piece
###############################################
mySmooth <- function(x, len, filter) {
  nn <- length(x)
  result <- rep(0, nn)
  counter <- 0
  nfilter <- length(filter)
  for(i in 1:length(len)) {
    tmpidx <- counter + (1:len[i])
    if(len[i]<nfilter)  ## filter long than time series, use average for all
      result[tmpidx] <- mean(x[tmpidx])
    else
      result[tmpidx] <- myfilter(x[tmpidx], filter)
    counter <- counter + len[i]
  }
  result
}


## a function to compute row variances
myvar <- function(x) {
  mx <- rowMeans(x)
  n <- dim(x)[2]
  rowSums((x-mx)^2)/(n-1)
}

########################################################################
## function to do shrinking
## this is modified JS shrinker, based on Ji's bioinformatics paper
#########################################################################
ji.shrinker <- function(x, v, truncate=FALSE) {
  I <- length(x)
  if(!truncate) {
    mx <- mean(x)
    S <- sum((x-mx)^2)
  } else { ## apply truncation
    cutoff <- quantile(x, truncate)
    yy <- x[x<cutoff]
    mx <- mean(yy)
    S <- sum((yy-mx)^2)
    I <- length(yy)
  }
  B <- (2/v)/(1+2/v)*(I-1)/I + 1/(1+2/v)*(2/v)*mx^2*(I-1)/S
  if(B>=1) ## B must be between 0 and 1
    B <- 1 
  
  result <- (1-B)*x + B*mx
  result
}

###########################################
## generate random variable from dirichlet distribution
###########################################
rdirichlet <- function(n, alpha) {
  l <- length(alpha)
  x <- matrix(rgamma(l * n, alpha), ncol = l, byrow = TRUE)
  sm <- x %*% rep(1, l)
  return(x/as.vector(sm))
}

#################################################
## find separate regions given probe locations
## This is useful because probes are not evenly
## spaced and there could be big gaps in between.
#################################################

findRegion <- function(chr, pos, sep=1000) {
  pos.diff <- abs(c(as.integer(0), diff(pos)))
  idx.jump <- which(pos.diff>sep)
  regions <- rbind(c(1, idx.jump), c(idx.jump-1, length(pos)))
  regions
}


##############################################################
## add two numbers in log domain. This works for vectors too.
##############################################################
Raddlog <- function(a, b) {
  result <- rep(0, length(a))
  idx1 <- a>b+200
  if(any(idx1))
    result[idx1] <- a[idx1]
  
  idx2 <- b>a+200
  if(any(idx2))
    result[idx2] <- b[idx2]
  
  idx0 <- !(idx1|idx2)
  if(any(idx0))
    result[idx0] <- a[idx0] + log1p(exp(b[idx0]-a[idx0]))
  result
}


##############################
## sum a vector in log domain
##############################
Rsumlog <- function(a) {
  s <- a[1]
  nn <- length(a)
  if(nn>1) {
    for(i in 2:length(a))
      s <- Raddlog(s, a[i])
  }
  s
}

#####################################
## variance for each row
#####################################
rowVars <- function(xx) {
  n <- ncol(xx)
  mm <- rowMeans(xx)
  rowSums((xx-mm)^2)/(n-1)
}

################################################
## function to compute log likelihoods
################################################
compute.density <- function(Y, s2, distn) {
  p0 <- 0.999
  nobs <- dim(Y)[1]
  ncond <- dim(Y)[2]
  mu0 <- distn[,1]
  mu1 <- distn[,2]
  tau0 <- distn[,3]
  tau1 <- distn[,4]

  ## compute densities
  lden1 <- lden0 <- Y
  for(i in 1:ncond) {
    ## f0
    if(!is.matrix(s2)) ## no probe level variance
      sd0 <- sqrt(tau0[i])
    else
      sd0 <- sqrt(s2[,i]+tau0[i])
    lden0[,i] <- dnorm(Y[,i], mu0[i], sd0)
    
    ## f1
    if(!is.matrix(s2))
      sd1 <- sqrt(tau1[i])
    else
      sd1 <- sqrt(s2[,i]+tau1[i])
    lden1[,i] <- dnorm(Y[,i], mu1[i], sd1)
    ## modify lden1 if there' no probe level variance
    if(!is.matrix(s2)) {
      thresh <- (mu0[i]/tau0[i]-mu1[i]/tau1[i]) / (1/tau0[i]-1/tau1[i])
      iii <- Y[,i] < thresh
      if(any(iii)) {
        ## make LR linear for those points - hopefully there're not too many
        k <- (mu1[i]-mu0[i])/sd0[1]^2
        lr <- log(lden1[,i]/lden0[,i])
        lr[iii] <- k*(Y[iii,i]-thresh)+min(lr)
        lden1[iii,i] <- lden0[iii,i] * exp(lr[iii])
      }
    }
  }
  
  ## use mixture model for background to remove positive outliers
  lden0 <- p0*lden0 + (1-p0)*lden1
  lden1 <- p0*lden1 + (1-p0)*lden0
  lden0 <- log(lden0); lden1 <- log(lden1)
  list(lden0=lden0, lden1=lden1)
}
  
