################################################
## wrapper functions 
################################################

###################################
## wraper function
## This sould read in a configuration file and:
## 1. Read configuration files
## 2. make options 
## 3. Read in data/indices
## 4. Run MVHMM.
###################################
jamie <- function(cfgfile, ngrp=1) {
  ## read in configuration file
  cfg <- read.config(cfgfile)
  ## read in data if not previous stored
  fn <- paste(cfg$data$WorkFolder, "/", cfg$data$Title, "-data.rda", sep="")
  if(file.exists(fn)) { ## load in data
    load(fn)
  } else {## read in data
    cat("Reading in data ... \n")
    if(cfg$data$Format=="cel") 
      data <- read.data.CEL(cfg)
    else
      data <- read.data.text(cfg)
  }
  
  ## select candidate regions or load previously saved ones
  fn <- paste(cfg$data$WorkFolder, "/", cfg$data$Title, "-candidates.rda", sep="")
  if(file.exists(fn)) { ## load in data
    load(fn)
  } else { ## select candidates       
    cat("Picking candidate regions ... \n")
    candidates <- select.candidate(data, cfg)
    fn <- paste(cfg$data$WorkFolder, "/", cfg$data$Title, "-candidates.rda", sep="")
    save(candidates, file=fn)
  }

  ## estimate parameters pi and q
  cat("Estimating parameters ... \n")
  param <- est.param(candidates, cfg)
  ## genome scan for peaks
  fn <- paste(cfg$data$WorkFolder, "/", cfg$data$Title, "-postprob.rda", sep="")
  if(file.exists(fn)) { ## load in data
    load(fn)
  } else { ## select candidates
    cat("Scanning genome for peaks ... \n")
    postprob <- gscan(data, candidates$distn, param$sita[2], param$q, cfg)
    save(postprob, file=fn)
  }
  
  ## detect and rank peaks 
  ncond <- dim(data$signal)[2]
  bumps.all <- vector("list", ncond)
  for(i in 1:ncond) {
    bumps.all[[i]] <- peakFinder(postprob, data$allchr, data$allpos, cfg,
                                 cutoff=0.05, cond=i, use.joint=TRUE)
  }
  if(!is.null(cfg$cel)) 
    names(bumps.all) <- names(cfg$cel)                                              
  if(!is.null(cfg$group))
    names(bumps.all) <- names(cfg$group)
  result <- list(peaks=bumps.all, param=param)

  ## estimate FDR
  if(cfg$peak$computeFDR==1) { ## estimate from data
    cat("estimating FDR from data ...\n")
    result0 <- negative.peaks(data, candidates$distn, param, cfg)
    for(i in 1:ncond) {
      fdr<- calc.FDR(result$peaks[[i]], result0$pool[[i]])
      result$peaks[[i]] <- data.frame(result$peaks[[i]], FDR=fdr)
    }
  } else { ## compute from local FDR (1-posterior probability)
    for(i in 1:ncond) {
      fdr <- cumsum(1-result$peaks[[i]]$postprob)/(1:length(result$peaks[[i]]$postprob))
      result$peaks[[i]] <- data.frame(result$peaks[[i]], FDR=fdr)
    }
  }

  ## trim the list according to FDR or posterior probability cutoff
  if(is.null(cfg$peak$FDRcutoff) & is.null(cfg$peak$Pvalue))
    ## user didn't specify either FDR cutoff or pvalue,
    ## assign  a default one
    cfg$peak$FDRcutoff <- 0.1

  ## filter peak list 
  if(!is.null(cfg$peak$FDRcutoff)) { ## use FDR cutoff
    for(i in 1:length(result$peaks)) {
      ii <- sum(result$peaks[[i]]$FDR<cfg$peak$FDRcutoff)
      result$peaks[[i]] <- result$peaks[[i]][1:ii,]
    }
  } else { ## use postprob cutoff
    for(i in 1:length(result$peaks)) {
      ii <- sum(result$peaks[[i]]$postprob>cfg$peak$Pvalue)
      result$peaks[[i]] <- result$peaks[[i]][1:ii,]
    }
  }
    
  ## save the rda file
#  fn <- paste(cfg$data$WorkFolder, "/", cfg$data$Title, "-result.rda", sep="")
#  save(result, file=fn)
  
  ## write out results to text files
  for(i in 1:length(result$peaks)) {
    fn <- paste(cfg$data$WorkFolder, "/", names(result$peaks)[i], "-peak.txt", sep="")
    ## format the results a little bit
    colnames(result$peaks[[i]])[c(7)] <- c("log(lr)")
    result$peaks[[i]]$postprob <- round(result$peaks[[i]]$postprob, 6)
    result$peaks[[i]]$FDR <- round(result$peaks[[i]]$FDR, 6)
    write.table(result$peaks[[i]], file=fn, sep="\t", row.names=FALSE, quote=FALSE)
  }
  cat("Done.\n")
}     

