/* function to scan the genome and compute posterior probabilities */
#include "jamie.h"
#include "util.h"
#include "gscan.h"

SEXP scan_engine(SEXP R_ylr, SEXP R_allpos, SEXP R_pi, SEXP R_q, 
		 SEXP R_bumplen, SEXP R_candPeakLen, SEXP R_maxGap) {
  double *ylr, lpi1, lpi0, *q, *lq1, *lq0, *ylr_cumsums, dtmp, dtmp2, *peaklr, joint_lr,
    *postprob, postprob_peak, *jointlr_cond, baseline;
  int i, j, *allpos, *bumplen, candPeakLen, nobs, *itmp, ncond, 
    nbumptype, *nprobes, minlen, maxGap;
  SEXP R_res, R_postprob, R_nprobes, R_peaklr;
  
  ylr=REAL(R_ylr);
  itmp=INTEGER(getAttrib(R_ylr, R_DimSymbol));
  nobs=itmp[0];
  ncond=itmp[1];

  q=REAL(R_q);
  lq1=(double *)R_alloc(ncond, sizeof(double));
  lq0=(double *)R_alloc(ncond, sizeof(double));
  for(i=0; i<ncond; i++) {
    lq1[i]=log(q[i]);
    lq0[i]=log(1-q[i]);
  }
  lpi1=log(*REAL(R_pi));
  lpi0=log(1-*REAL(R_pi));
  allpos=INTEGER(R_allpos);
  bumplen=INTEGER(R_bumplen);
  candPeakLen=*INTEGER(R_candPeakLen);
  nbumptype=length(R_bumplen);
  maxGap=*INTEGER(R_maxGap);
  
  /* allocate memory for output, which is a list of 2 fields:
     1. A matrix of posterior probabilities. The last column is Pr(Z=1|data).
     2. Number of probes contained in a peak if the peak is initiated at the current probe. */
  PROTECT(R_peaklr=allocMatrix(REALSXP, nobs, ncond));
  peaklr=REAL(R_peaklr);
  PROTECT(R_postprob=allocMatrix(REALSXP, nobs, ncond+1));
  postprob=REAL(R_postprob);
  PROTECT(R_nprobes=allocVector(INTSXP, nobs));
  nprobes = INTEGER(R_nprobes);

  /* 1. compute cumsum of the likelihood ratio*/
  ylr_cumsums = (double *) R_alloc(nobs*ncond, sizeof(double));
  cumsum2(ylr, nobs, ncond, ylr_cumsums);
  
  /* 2. get the number of probes contained in a peak if the peak is initiated at the current probe */
  nitem_bin_left(allpos, nobs, candPeakLen, nprobes);

  /* 3. start to scan the genome */
  minlen=candPeakLen/2;
  /*  peaklr=(double *)R_alloc(ncond, sizeof(double)); */
  jointlr_cond=(double *)R_alloc(ncond, sizeof(double));
  for(i=0; i<nobs-1; i++) {
    /* if don't have enough length to initiation a peak, go to next */
    if(nprobes[i]<=3 | allpos[i+nprobes[i]-1]-allpos[i] < minlen) {
      for(j=0; j<ncond; j++) {
	postprob[j*nobs+i]=0;
	peaklr[j*nobs+i]=-200;
      }
      postprob[ncond*nobs+i]=0;
      continue;
    }
    /* compute lr for each condition, then marginalize likelihood on each condition
       and compute joint likelihood for the candidate region */
    dtmp=0;
    for(j=0; j<ncond; j++) {
      if(i==0)
	baseline=0;
      else
	baseline=ylr_cumsums[j*nobs+i-1];
      peaklr[j*nobs+i]=calcOnelr(ylr_cumsums+j*nobs+i, allpos+i, nprobes[i], bumplen, nbumptype, baseline);
      jointlr_cond[j]=logspace_add(peaklr[j*nobs+i]+lq1[j], lq0[j]); /* this will be used later */
      dtmp = dtmp + jointlr_cond[j];

    }
    /* compute posterior probability for being a binding site */
    dtmp2 = dtmp + lpi1;
    postprob_peak=exp(dtmp2-logspace_add(dtmp2, lpi0));
    postprob[ncond*nobs+i]=postprob_peak;
    /* postprob for each condition */
    for(j=0; j<ncond; j++) {
      postprob[j*nobs+i]=exp(peaklr[j*nobs+i]+lq1[j]-jointlr_cond[j]);
    }
  }

  /* create return object */
  PROTECT(R_res = allocVector(VECSXP, 3));
  SET_VECTOR_ELT(R_res, 0, R_postprob);
  SET_VECTOR_ELT(R_res, 1, R_nprobes);
  SET_VECTOR_ELT(R_res, 2, R_peaklr);
  /*  PROTECT(names = allocVector(VECSXP, 2));
  SET_VECTOR_ELT(names, 0, mkChar("postprob"));
  SET_VECTOR_ELT(names, 1, mkChar("nprobes"));
  setAttrib(R_res, R_DimNamesSymbol, dimnames); */

  UNPROTECT(4);
  return(R_res);
}

/* compute the number of probes included in ws bps if a peak is initiated from the current probe */
void nitem_bin_left(int *allpos, int nobs, int ws, int *nprobes) {
  int i, j, currenti=1;

  for(i=0; i<nobs; i++) {
    for(j=currenti; j<nobs; j++) {
      if(allpos[j]-allpos[i]>ws | j==nobs-1) {
	if(j==nobs-1)
	  nprobes[i] = j-i+1;
	else 
	  nprobes[i] = j-i;
	currenti = j;
	break;
      }
    }
  }
  nprobes[nobs-1] = 0; /* the last one */
}
